import java.awt.*;
import java.net.URL;

/**
 * SmartMap is an applet that superimposes text over an
 * image depending on where the mouse is located. It uses
 * the same COORDS and HREF attributes of the HTML extensions
 * for client-side image maps, and adds a TEXT attribute for
 * a cutline that is displayed atop the image until the mouse
 * is moved out of the cutline's mapping. Copyright 1996 Rogers
 * Cadenhead. You can use this applet freely on your own web
 * pages, and use any part of it in your own Java programming
 * with no restrictions.
 * @author Rogers Cadenhead
 * @version 1.0
 */
public class SmartMap extends java.applet.Applet implements Runnable {

    protected String getParam(String name, String defaultValue) {
        String s = getParameter(name);
        return (s == null) ? defaultValue : s;
    }

    Thread runner;
    Image img;
    Image offscreen;
    Graphics offG;
    String font;
    int fontSize, fontStyle;
    Font appFont;
    String boxText[] = new String[30];
    URL boxURL[] = new URL[30];
    Cutline boxLoc[] = new Cutline[30];
    int numBoxes = 0;
    int mustDraw = -1;
    int boxX, boxY;
    int xctr, yctr;

    public void start() {
        if (runner == null) {
            runner = new Thread(this);
            runner.start();
        }
    }

    public void stop() {
        if (runner != null) {
            runner.stop();
            runner = null;
        }
    }

    public void init() {
        // find center of applet
        xctr = size().width / 2;
        yctr = size().height / 2;

        // set up offscreen buffer
        offscreen = createImage(size().width, size().height);
        offG = offscreen.getGraphics();

        // load image
	  String imgName = getParam("image", "none");
        try {
	    img = getImage(getDocumentBase(), imgName);
        } catch (Exception e) {
            System.out.println(e);
	}	

        // get font parameters
        font = getParam("font", "Arial");
        fontSize = Integer.parseInt(getParam("fontSize", "12"));
        String style = getParam("fontstyle", "none");
        fontStyle = 0;
        if (style == "BOLD") fontStyle += Font.BOLD;
        else if (style == "ITALIC") fontStyle += Font.ITALIC;
        appFont = new Font(font, fontStyle, fontSize);
	  
        // load text array
        int node = 0;
        String nodeText = getParam("text" + (node + 1), "none");
	  while (nodeText != "none" && node < 30) {
	    boxText[node] = nodeText;
	    // get URL
            try {
	        boxURL[node] = null;
                String urlText = getParam("href" + (node + 1), "none");
	        if (!urlText.equals("none")) {
                    boxURL[node] = new URL(getDocumentBase(), urlText);
	        }
            } catch (Exception e) {
	        System.out.println(e);
            }

            // load coordinates
            String c = getParam("coords" + (node + 1), "none");
            int cl = c.length();
            if (!c.equals("none")) {
                int c1 = c.indexOf(",");
                int c2 = c.indexOf(",",
                    (c1 + 1 <= cl) ?  c1 + 1 : cl);
                int c3 = c.indexOf(",",
                    (c2 + 1 <= cl) ?  c2 + 1 : cl);
                boxLoc[node] = new Cutline();
                boxLoc[node].x = Integer.parseInt(c.substring(0, c1));
                boxLoc[node].y = Integer.parseInt(c.substring(c1+1, c2));
                boxLoc[node].width = Integer.parseInt(c.substring(c2+1, c3))
                    - boxLoc[node].x;
                boxLoc[node].height = Integer.parseInt(c.substring(c3+1, cl))
                    - boxLoc[node].y;
                numBoxes += 1;
            }               
	      nodeText = getParam("text" + (++node+1), "none");
            repaint();
        }
    }

    public void paint(Graphics g) {
        offG.drawImage(img, 0, 0, this);
        if (mustDraw != -1) {
            // set box coordinates
            boxY = boxLoc[mustDraw].y + boxLoc[mustDraw].height;
            int ht = boxLoc[mustDraw].findHeight(offG,
                boxText[mustDraw], appFont, size().width / 4);
            if (boxY + ht > size().height) {
                boxY = boxLoc[mustDraw].y - ht;
            }
            boxX = boxLoc[mustDraw].x - 
                (size().width / 4 - boxLoc[mustDraw].width) / 2;
            if (boxX < 0) boxX = 1;
            if ((boxX + (size().width / 4)) > size().width)
                boxX = size().width - (size().width / 4) - 1;

            // draw box
            boxLoc[mustDraw].draw(offG, boxX, boxY,
                boxText[mustDraw], appFont, size().width / 4);
        }
        g.drawImage(offscreen, 0, 0, this);
    }
  
    public void update(Graphics g) {
        paint(g);
    }

    public void run() {
        /* while (true) {
            repaint();
            try { Thread.sleep(1000); }
	    catch (InterruptedException e) { }
        } */
    }

    public boolean mouseMove(Event evt, int x, int y) {
        mustDraw = inZone(numBoxes, x, y);
        if (mustDraw != -1 && boxURL[mustDraw] != null)
            showStatus("" + boxURL[mustDraw]);
        else
            showStatus("Applet SmartMap running (" + x +
                ", " + y + ")");
        repaint();
	  return false;
    }

    public boolean mouseDown(Event evt, int x, int y) {
        if (mustDraw != -1 && boxURL[mustDraw] != null)
            getAppletContext().showDocument(boxURL[mustDraw]);
        return true;
    }

    public int inZone(int numBoxes, int x, int y) {
        for (int zone = 0; zone < numBoxes; zone++)
            if (boxLoc[zone].inside(x, y)) return zone;
        return -1;
    }
}
